use crate::extractors::family_extractor::FamilyInPath;
use crate::models::{Accommodation, AccommodationID, FamilyID, Membership};
use crate::services::accommodations_list_service;
use actix_web::dev::Payload;
use actix_web::{FromRequest, HttpRequest};
use serde::Deserialize;
use std::ops::Deref;

#[derive(thiserror::Error, Debug)]
enum AccommodationExtractorErr {
    #[error("Accommodation {0:?} does not belong to family {1:?}!")]
    AccommodationNotInFamily(AccommodationID, FamilyID),
}

#[derive(Debug)]
pub struct FamilyAndAccommodationInPath(Membership, Accommodation);

impl FamilyAndAccommodationInPath {
    async fn load_accommodation_from_path(
        family: FamilyInPath,
        accommodation_id: AccommodationID,
    ) -> anyhow::Result<Self> {
        let accommodation = accommodations_list_service::get_by_id(accommodation_id).await?;
        if accommodation.family_id() != family.family_id() {
            return Err(AccommodationExtractorErr::AccommodationNotInFamily(
                accommodation.id(),
                family.family_id(),
            )
            .into());
        }

        Ok(Self(family.into(), accommodation))
    }
}

impl Deref for FamilyAndAccommodationInPath {
    type Target = Accommodation;

    fn deref(&self) -> &Self::Target {
        &self.1
    }
}

impl FamilyAndAccommodationInPath {
    pub fn membership(&self) -> &Membership {
        &self.0
    }

    pub fn to_accommodation(self) -> Accommodation {
        self.1
    }
}

#[derive(Deserialize)]
struct AccommodationIDInPath {
    accommodation_id: AccommodationID,
}

impl FromRequest for FamilyAndAccommodationInPath {
    type Error = actix_web::Error;
    type Future = futures_util::future::LocalBoxFuture<'static, Result<Self, Self::Error>>;

    fn from_request(req: &HttpRequest, _payload: &mut Payload) -> Self::Future {
        let req = req.clone();
        Box::pin(async move {
            let family = FamilyInPath::extract(&req).await?;

            let accommodation_id = actix_web::web::Path::<AccommodationIDInPath>::from_request(
                &req,
                &mut Payload::None,
            )
            .await?
            .accommodation_id;

            Self::load_accommodation_from_path(family, accommodation_id)
                .await
                .map_err(|e| {
                    log::error!("Failed to extract accommodation ID from URL! {e}");
                    actix_web::error::ErrorNotFound("Could not fetch accommodation information!")
                })
        })
    }
}
